/*
 * Christophe Lucas <clucas@rotomalug.org>
 *
 * Setup Intel 8259 PIC
 */

#include <video/bochs.h>
#include <video/x86_video.h>
#include <testos/types.h>
#include <testos/ioports.h>
#include <testos/errno.h>
#include <x86/i8259.h>

ret_t i8259_setup_pic()
{
	/* ICW1 :
 	 * Level, call add intervall=8, cascade, icw4   
	 */
	outb( 0x11, PIC1_BASE_ADD );
	outb( 0x11, PIC2_BASE_ADD );

	/* 
         * ICW2 :
	 *  - 0x20 = (decimal)32 : On dcale les interruptions dans le vecteur 
	 * d'interruptions de 32 car les 32 premire ssont des exceptions.
	 * - 0x28 : 0x20 + 0x8 : PIC2 ets lui dcal aussi de 
	 * 0x20 + 8'les 8 premire du PIC1 
	 */
	outb( 0x20, PIC1_BASE_ADD+1 );
	outb( 0x28, PIC2_BASE_ADD+1 );

	/*
	 * ICW3 : 	
	 * Ox04 : IRQ2 connect au pic esclave
	 * 0x02 : 7-3 : unused ; 2-0: salveid (ici on prendr salve1: premire esclave)
	 */
	outb( 0x04, PIC1_BASE_ADD+1 );
	outb( 0x02, PIC2_BASE_ADD+1 );
	 
	/*
	 * ICW4 : On a dit  ICW1 qu'on avait un ICW4, alors on y va :
	 * Seulement le mode 8086 est mis  1.
	 */
	outb(0x01, PIC1_BASE_ADD+1);
	outb(0x01, PIC2_BASE_ADD+1);

	return OK;
}

ret_t i8259_disable_irq_line( ui8_t irq )
{
	if ( irq > 8 ) {
		outb( inb(PIC1_BASE_ADD+1) | (1<<irq) , PIC1_BASE_ADD+1);
	} else {
		outb( inb(PIC2_BASE_ADD+1) | (1<<(irq-8)), PIC2_BASE_ADD+1);
	}	
	return OK;
}

ret_t i8259_enable_irq_line( ui8_t irq )
{
	if ( irq > 8 ) {
		outb( inb(PIC1_BASE_ADD+1) & ~(1<<irq) , PIC1_BASE_ADD+1);
	} else {
		outb( inb(PIC2_BASE_ADD+1) & ~(1<<(irq-8)), PIC2_BASE_ADD+1);
	}	
	return OK;
}

ui8_t i8259_get_imr_master( void )
{
	return inb(PIC1_BASE_ADD+1 );
}

ui8_t i8259_get_imr_slave( void )
{
	return inb(PIC2_BASE_ADD+1);
}

