/*
 * Christophe Lucas <clucas@rotomalug.org>
 *
 * Setup Intel 8259 PIC
 */

#include <video/bochs.h>
#include <video/x86_video.h>
#include <testos/types.h>
#include <testos/ioports.h>
#include <testos/errno.h>
#include <x86/processor.h>

static inline unsigned int verify_cpuid(unsigned int op)
{
        unsigned int eax;
	bochs_printf("op=%x\n", op);
        __asm__ ("cpuid"
		: "=a" (eax)
		: "m" (op)
		: "bx", "cx", "dx");
	bochs_printf("eax=%x\n", eax);
        return eax;
}



int processor_identification()
{
	ui32_t cpuid_flags;
	processor_t processor;

	/* 0x8000000 = 1 << 21 */
	if (verify_cpuid(0x8000000) < 0x80000004) {
		/* Extented Function Information */
		bochs_printf("EFI is not implemented.\n");
		printk("EFI is not implemented.\n");
	} else {
		bochs_printf("Here we go cpuid intr is valid!\n");
	}

	/*printk("Processor identification:\n");
	printk("stepping : 0x%x\n", PROC_GET_STEPPING(processor));
	printk("model    : 0x%x\n", PROC_GET_MODEL(processor));
	printk("family   : 0x%x\n", PROC_GET_FAMILY(processor));
	printk("type     : 0x%x\n", PROC_GET_TYPE(processor));
	*/
	__asm__ __volatile__ ("cpuid"
		      : "=d" (cpuid_flags)
		      : "a" (1));

	bochs_printf("processor type=%x\n", cpuid_flags);
	return 0;
}

